/* @Copyright (C) 2002, Oracle Corporation. All rights reserved. */

create or replace package body wwwcp_api_registry is

   /**
    * @headcom
    *
    * Web Clipping Provider Registry APIs providing storage for Web Clipping
    * Definitions.
    */

   /** 
    * This constant is the upper-case representation of FALSE.
    */
    WCR_FALSE_U constant varchar2(1) := 'N';

   /** 
    * This constant is the upper-case representation of FALSE.
    */
    WCR_FALSE_L constant varchar2(1) := 'n';

   /** 
    * This constant is the upper-case representation of TRUE.
    */
    WCR_TRUE_U constant varchar2(1) := 'Y';

   /** 
    * This constant is the lower-case representation of TRUE.
    */
    WCR_TRUE_L constant varchar2(1) := 'y';
    
   /**
    * This API inserts an entry into the instance table to denote that a
    * customization of a Portlet has been created. If the customization is done
    * with the default user of WCR_DEFAULT_USER, then it means this is the 
    * first time that the Portlet has been instantiated, as reflected by the
    * short-circuited above method.
    *
    * It works for both Phase I and Phase II in different ways. 
    * 
    * In Phase I, the instantiation of a Portlet happens before the Clipping
    * Definition has been created. 
    * 
    * In Phase II, the instantiation of a Portlet happens after the Clipping
    * Definition has been created. 
    */
    function insert_customization(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    ) return integer is
        l_cust_id integer;
        l_clip_id integer;
    begin
       
        -- Get a new value of a customization.
        select wwwcp_cust_id_sequence.nextval
        into l_cust_id
        from dual;               
        
        -- Put some value in the clip id as well
        -- If we are in the Phase I, then we want to inherit it from
        -- any default clip that existed.
        if p_portlet_id = WCR_GENERIC_PORTLET then 
           l_clip_id := NULL;
        -- Phase II, the clip id maps to portlet id
        elsif is_number(p_portlet_id) = 1 then
           -- In Phase II, the clip id is the portlet id and they are numbers
           l_clip_id := to_number(p_portlet_id);
        -- Wireless case, the portlet id is not a number
        else 
           l_clip_id := NULL;
        end if;
        
        -- Insert into the customization_map table because of foreign key
        -- If we're in phase I, then we don't have a clip id but in Phase II
        -- we do, which is the portlet id.      
        insert into wwwcp_customization_map$ (
            cust_id, 
            clip_id
        )
        values (
            l_cust_id,
            l_clip_id
        );
        
        -- We are guaranteed that there are no duplicates for this table
        insert into wwwcp_customization$ (
            provider_id,
            portlet_id,
            portlet_instance_id,
            portal_user_id,
            cust_id
        )
        values (
            p_provider_id, 
            p_portlet_id, 
            p_portlet_instance_id, 
            p_portal_user_id,
            l_cust_id
        );
        
        -- In Phase I, we will usually call the twin of this method with
        -- three parameters to create a new cust id with the '-' user.
        -- Then after this insertion, a clip will be created with the
        -- default customization represented by this row and will update
        -- the customization_map table. 
        return l_cust_id;
        
    end insert_customization;
    
   /**
    * This API creates an instance of the Portlet as it is born after dropping
    * itself onto a Portal Page. In essence a default customization is being
    * created so it calls the insert_customization method with WCR_DEFAULT_USER
    * as the user that is doing so.
    */
    procedure create_instance(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    ) is
        l_cust_id integer;
    begin
      -- In phase I, only default user can create an instance
      if p_portlet_id = WCR_GENERIC_PORTLET then
        l_cust_id := insert_customization(
           p_provider_id         => p_provider_id,
           p_portlet_id          => p_portlet_id,
           p_portlet_instance_id => p_portlet_instance_id,
           p_portal_user_id      => WCR_DEFAULT_USER
       );
      else
       -- In phase II, any user can create an instance, which maps to an
       -- existing clip. The map is going to be updated in insert_customization
        l_cust_id := insert_customization(
           p_provider_id         => p_provider_id,
           p_portlet_id          => p_portlet_id,
           p_portlet_instance_id => p_portlet_instance_id,
           p_portal_user_id      => p_portal_user_id
        );
      end if;

    end create_instance;
    
   /**
    * This API looks up the customization table for a customization id given
    * a foursome key of Provider Id, Portlet Id, Portlet Instance Id, and 
    * Portal User Id. This also allows the caller to insert a default
    * customization if it is not there.
    */
    function lookup_customization(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2,
        p_create              in boolean
    ) return integer is
       l_cust_id integer;
    begin
       
        -- Simple select into clause.
        select cust.cust_id 
        into l_cust_id
        from wwwcp_customization$ cust
        where 
            cust.provider_id         = p_provider_id         and
            cust.portlet_id          = p_portlet_id          and
            cust.portlet_instance_id = p_portlet_instance_id and
            cust.portal_user_id      = p_portal_user_id;
        return l_cust_id;
    
    exception
       
        when NO_DATA_FOUND then
          
            if p_create then
                return insert_customization(
                    p_provider_id         => p_provider_id,
                    p_portlet_id          => p_portlet_id,
                    p_portlet_instance_id => p_portlet_instance_id,
                    p_portal_user_id      => p_portal_user_id
                );
            else
               return NULL;
            end if;
            
    end lookup_customization;
    
   /**
    * This API is a helper function that tells if a certain VARCHAR2 is
    * a number and therefore can be converted into a number. 
    * 
    * It returns a value of 1 if the given string is a number and a 
    * value of 0 if not.
    */
    function is_number(p_string in varchar2) return integer is
        dummy number(38,4);
    begin
        select to_number(p_string) into dummy from dual;
        return 1;
    exception
        when others then
            return 0;
    end is_number;

   /**
    * This API performs a lookup the Clip Id associated with the given mapping.
    * In Phase I, this foursome mapping comprised of the provider id, portlet 
    * id, portlet instance id and portal user. In Phase II, there is really no
    * mapping; the clip id is really the portlet_id.
    *
    * This method does not default to use the WCR_DEFAULT_USER when nothing
    * is found as the clip created using "Edit Defaults" would have that as the
    * portal user..
    */
    function lookup_clip_id_without_def(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    ) return integer is
        l_query_clip_id integer;
        l_query_cust_id integer;
    begin
        -- Check for Phase I and Phase II. 
        -- Only Phase I has the generic Portlet.
        if  p_portlet_id <> WCR_GENERIC_PORTLET and
            p_portlet_id <> '-' || WCR_GENERIC_PORTLET then
            if is_number(p_portlet_id) = 1 then 
                -- This is for Portal Phase II, in which the clip id maps 
                -- to portlet id.
                return to_number(p_portlet_id);
            end if;
        end if;
        
        -- Lookup the customization for this foursome
        l_query_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => FALSE
        );
        
        -- Use that cust id to find the clip id. This would return NULL
        -- when the search is only supposed to return for the default.
        select m.clip_id
        into l_query_clip_id 
        from wwwcp_customization_map$ m
        where m.cust_id = l_query_cust_id;
        
        -- TODO: Check for NULL and throw a NO_DATA_FOUND
        --       For now just have the call to this check
        
        -- Return the id looked up.
        -- Unlike the previous method, here we let go of the exception.
        return l_query_clip_id;

    end lookup_clip_id_without_def;

   /**
    * This API performs a lookup the Clip Id associated with the given
    * provider id, portlet id, portlet instance id and portal user, and will
    * default to use the WCR_DEFAULT_USER which is used for clipping
    * definitions created using "Edit Defaults", when one is not found 
    * associated with the passed in portal user id.
    */
    function lookup_clip_id_with_def(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    ) return integer is
        l_clip_id integer;
    begin
        -- This would throw us an exception.
        -- Exceptions happen when nothing is found. Then we must default 
        -- to use the default user which is there by edit default.
        l_clip_id := lookup_clip_id_without_def(
            p_provider_id, 
            p_portlet_id, 
            p_portlet_instance_id, 
            p_portal_user_id
        );
        
        -- Check for NULL
        if l_clip_id is NULL then
            return lookup_clip_id_without_def(
                p_provider_id         => p_provider_id, 
                p_portlet_id          => p_portlet_id,
                p_portlet_instance_id => p_portlet_instance_id, 
                p_portal_user_id      => WCR_DEFAULT_USER
            );
        else
           return l_clip_id;
        end if;
        
    exception
        when NO_DATA_FOUND then
            return lookup_clip_id_without_def(
                p_provider_id         => p_provider_id, 
                p_portlet_id          => p_portlet_id,
                p_portlet_instance_id => p_portlet_instance_id, 
                p_portal_user_id      => WCR_DEFAULT_USER
            );
    end lookup_clip_id_with_def;
    
   /**
    * This API returns the list of clip ids that have a title or description 
    * that contains the passed in String criterion.
    */
    function lookup_clip_ids_with_contains(
        p_str_contains in varchar2
    )
    return clipping_metadata_cursor is
        l_cv_clip_ids clipping_metadata_cursor;
    begin
        open l_cv_clip_ids for

            -- This is for a simple containment query
            select c.clip_id, c.clip_title, c.clip_description 
            from wwwcp_clipping$ c
            where
                c.clip_title like '%'||p_str_contains||'%' or
                c.clip_description like '%'||p_str_contains||'%';
            
        -- Return the Clip Ids
        return l_cv_clip_ids;
    end;
    
   /**
    * This API removes the clip identified by a specified portlet instance
    * and the specific portal user id passed in. Note this function does not
    * cascade the remove to the customizations.
    */
    function remove_clip(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    ) return integer is
        l_query_clip_id integer;
        l_clip_id_count integer;
    begin

        -- Find the clip id that the four keys map to. The implementation of
        -- the lookup_clip_id_without_def will take into account the
        -- differences between Phase I and Phase II.
        l_query_clip_id := lookup_clip_id_without_def (
            p_provider_id, 
            p_portlet_id, 
            p_portlet_instance_id, 
            p_portal_user_id
        );
        
        -- If we are the default guy, then go ahead and delete the clip
        -- Otherwise, we must check to see if we are the owner of the clip
        select count(*) 
        into l_clip_id_count 
        from wwwcp_customization_map$ m
        where m.clip_id = l_query_clip_id;
        
        -- If the clip id count is 1 that means the portal user is the owner
        -- of the clip. Otherwise, if we decide to remove the whole clip as
        -- the edit default user (don't know when this happens) then we also
        -- can delete.
        if l_clip_id_count = 1 or p_portal_user_id = WCR_DEFAULT_USER then
            -- Delete from the clipping table cascade with same clipid.
            delete from wwwcp_clipping$ c where c.clip_id = l_query_clip_id;
        end if;

        -- Delete from the clip element path
        delete from wwwcp_element_path$ ep where ep.clip_id = l_query_clip_id;
        
        -- Return the clip_id deleted
        return l_query_clip_id;
        
    end remove_clip;
  
   /**
    * This API removes the clip identified by a specified portlet instance
    * and the specific portal user id passed in. In addition, this propagates
    * the removal down to all the customizations of that clip.
    */
    function remove_clip_cascade(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    ) return integer is
        l_query_clip_id integer;
    begin
       
       RETURN remove_clip(
         p_provider_id, 
         p_portlet_id, 
         p_portlet_instance_id,
         p_portal_user_id);
       
    end remove_clip_cascade;
  
    procedure update_customization(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2,
        p_clip_id             in varchar2
    ) is
        l_cust_id integer;
    begin      
       
        -- Find what the customization id was from previous
        -- Or create one if not found.
        l_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => TRUE
        );
       
        -- Update the customization map by updating it an entry 
        -- pointing to the clip id. In the event that this is the first
        -- time that we're doing the insert, the old clip_id value would
        -- have been NULL, which means the next update would work and
        -- would set up the foreign key constraint.
        --
        -- This would never fail if the above succeeded because of the foreign
        -- key constraint. By virtue of the fact that there is a cust id from
        -- the customization table, the customization map table must have a
        -- row with that being the cust id. So we just do the update there
        --
        -- A previous remove (non-cascade) would have made the entry that
        -- is with this cust id have a NULL value for the clip id.
        update wwwcp_customization_map$ m
        set m.clip_id = p_clip_id 
        where m.cust_id = l_cust_id;

    end update_customization;
    
   /**
    * This API inserts a Clipping definition in our Phase I/II implementation,
    * mapping it to the four keys that uniquely identify a clipping id, and
    * returns the newly generated Clip Id.
    *
    * customization id and therefore the clip id to do the insertion with
    */
    function insert_clip(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    ) return integer is 
        l_query_clip_id integer;
        l_query_cust_id integer;
    begin
        
        -- Get ready to catch an exception.
        -- Conceptually, after this block, the database is in pristine
        -- state so that whatever l_query_clip_id may be, (either the old
        -- one that we will have deleted, or the new one that we asked the
        -- sequence for) an insertion can be made.
        begin
           
            -- First find out if we need to do an update instead of an insert
            l_query_clip_id := lookup_clip_id_without_def(
                p_provider_id         => p_provider_id, 
                p_portlet_id          => p_portlet_id,
                p_portlet_instance_id => p_portlet_instance_id, 
                p_portal_user_id      => p_portal_user_id
            );
                       
            -- In the event of a rebind after the url parameters are added
            -- the clip id will get returned for the Page Viewer because he
            -- now has a row in the mapping table for the same clip id.
            -- To take care of this, we must know what the 
            
        exception
            WHEN NO_DATA_FOUND then
            l_query_clip_id := NULL; 
        end;
                     
        -- Remove the customizations of the clip
        -- Do this no matter if we're going to insert or update because in
        -- both cases, the locator table and url parameter table doesn't really
        -- apply to the same clip anymore.
        -- In phase I, this clip only maps to one portlet instance.
        -- In Phase II, this clip maps all the portlet instances created from 
        -- this clip.
        remove_clip_customization(
              p_provider_id         => p_provider_id,
              p_portlet_id          => p_portlet_id,
              p_portlet_instance_id => p_portlet_instance_id,
              p_portal_user_id      => p_portal_user_id
        );
                
        -- If nothing is returned, we simply do the regular insertion
        if l_query_clip_id is NULL then

            -- Get the clip id to insert from the sequence.
            select wwwcp_clip_id_sequence.nextval
            into l_query_clip_id
            from dual;
           
            -- insert the clip with id
            insert_clip(
                p_clip_id             => l_query_clip_id,
                p_clip_path           => p_clip_path,
                p_clip_signature      => p_clip_signature,
                p_clip_description    => p_clip_description,
                p_clip_title          => p_clip_title,
                p_clip_timeout        => p_clip_timeout,
                p_clip_max_tries      => p_clip_max_tries,
                p_clip_expiration     => p_clip_expiration,
                p_clip_rebindable     => p_clip_rebindable
            );
        
        else
               
            -- Remove the browse url and default url parameter table
            delete from wwwcp_browse_url$ bu
            where bu.clip_id = l_query_clip_id;
                
            delete from wwwcp_default_url_parameter$ dup
            where dup.clip_id = l_query_clip_id;
                
            -- Delete from the clip element path
            delete from wwwcp_element_path$ ep 
            where ep.clip_id = l_query_clip_id;        

            -- Update the clip table
            update wwwcp_clipping$ c
            set
                c.clip_stylesheet  = NULL,
                c.clip_description = p_clip_description,
                c.clip_title       = p_clip_title,
                c.clip_timeout     = p_clip_timeout,
                c.clip_max_tries   = p_clip_max_tries,
                c.clip_expiration  = p_clip_expiration,
                c.clip_rebindable  = p_clip_rebindable,
                c.clip_path        = p_clip_path,
                c.clip_signature   = p_clip_signature
            where c.clip_id = l_query_clip_id;
                
        end if;
               
        -- If an entry does not exist in the customization map table for the
        -- cust id / clip id pair, then either update the one for the current
        -- cust id by doing a lookup, or insert a brand new one as in the case
        -- of a rebinded clip.
  
        -- This is the creation time for phase II, in which case, 
        -- portlet id is the same as clip id
        -- TODO: Ask Wei why this is that way???
        if p_portlet_id = '-' || WCR_GENERIC_PORTLET then 
          update_customization(
              p_provider_id         => p_provider_id,
              p_portlet_id          => l_query_clip_id,
              p_portlet_instance_id => p_portlet_instance_id,
              p_portal_user_id      => p_portal_user_id,
              p_clip_id             => l_query_clip_id
          );

        else
          update_customization(
              p_provider_id         => p_provider_id,
              p_portlet_id          => p_portlet_id,
              p_portlet_instance_id => p_portlet_instance_id,
              p_portal_user_id      => p_portal_user_id,
              p_clip_id             => l_query_clip_id
          );
        end if;

        
        -- Just return what has been inserted
        return l_query_clip_id;
    
    end insert_clip;

   /**
    * This API inserts a Clipping definition in the Phase II implementation. 
    * It happens during the creation of a Portlet using the Web Clipping type.
    * Naturally, we do not have the four keys to put in the mapping table.
    * Thus, we need to simply insert and return the clip id.
    */
    procedure insert_clip(
        p_clip_id             in integer,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    ) 
    is 
    begin
        -- Insert into the actual clipping table with the real values.
        insert into wwwcp_clipping$ c (
            clip_id,
            clip_stylesheet,
            clip_description,
            clip_title,
            clip_timeout,
            clip_max_tries,
            clip_expiration,
            clip_rebindable,
            clip_path,
            clip_signature
        )
        values ( 
            p_clip_id,
            NULL,
            p_clip_description,
            p_clip_title,
            p_clip_timeout,
            p_clip_max_tries,
            p_clip_expiration,
            p_clip_rebindable,
            p_clip_path,
            p_clip_signature
        );

    end insert_clip;
    
   /**
    * This API inserts a URL parameter for a particular clip for a particular
    * portal user id.
    */
    procedure insert_url_parameter(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_url_index           in integer,
        p_param_name          in varchar2,
        p_param_value         in varchar2
    )
    is
        l_cust_id integer;
    begin
       
        -- Lookup the customization id from the customization table.
        -- If no customization exists, it will generate one for you
        -- Turn the create to true because we do need that entry there
        l_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => TRUE
        );
        
        -- This call will result in DUP_VAL_ON_INDEX if one is already there.
        -- We supply the EXCEPTION block that follows to take care of this.
        insert into wwwcp_url_parameter$ u (
            cust_id,
            url_index,
            param_name,
            param_value
        )
        values ( 
            l_cust_id,
            p_url_index,       
            p_param_name,
            p_param_value
        );

    exception
        -- When there is a unique key constraint, turn it into update.
        when DUP_VAL_ON_INDEX then
        update wwwcp_url_parameter$ u 
        set u.param_value = p_param_value 
        where 
            u.cust_id    = l_cust_id    and
            u.url_index  = p_url_index  and
            u.param_name = p_param_name;

    end insert_url_parameter;
  
   /**
    * This API inserts a new browse url that is part of the 
    * original definition of the Clipping.
    */
    procedure insert_browse_url(
       p_clip_id           in integer,
       p_url_index         in integer,
       p_url               in varchar2,
       p_method            in varchar2,
       p_req_enc           in varchar2,
       p_auth_type         in varchar2,
       p_auth_realm        in varchar2,
       p_username          in varchar2,
       p_password          in varchar2,
       p_obfuscation_key   in varchar2
    )
    is
        l_password varchar2(128);
    begin
        -- Encrypt the password before sending it out; using itself as an
        -- indicator for NULL and therefore no op.
        l_password := encrypt_if_needed(
            WCR_TRUE_L, 
            p_obfuscation_key, 
            p_password
        );

        -- Perform a simple insert with the encrypted password
        insert into wwwcp_browse_url$ ( 
            clip_id, 
            url_index, 
            url, 
            method,
            req_enc,
            auth_type, 
            auth_realm,
            username, 
            password
        )
        values ( 
            p_clip_id, 
            p_url_index, 
            p_url, 
            p_method,
            p_req_enc,
            p_auth_type, 
            p_auth_realm,
            p_username, 
            l_password
        );
    end insert_browse_url;

   /**
    * This API inserts a new url param that is part of the original definition
    * of the Clipping. 
    */
    procedure insert_default_url_param(
       p_clip_id           in integer,
       p_url_index         in integer,
       p_param_name        in varchar2,
       p_param_value       in varchar2,
       p_param_usage       in varchar2,
       p_encrypted         in varchar2,
       p_customizable      in varchar2,
       p_customizable_name in varchar2,
       p_obfuscation_key   in varchar2
    )
    is
        l_param_value varchar2(2048);
    begin
        l_param_value := encrypt_if_needed(
            p_encrypted, 
            p_obfuscation_key, 
            p_param_value
        );
     
        -- Use the local variable to insert.
        insert into wwwcp_default_url_parameter$ ( 
            clip_id, 
            url_index, 
            param_name, 
            param_value, 
            param_usage, 
            encrypted, 
            customizable, 
            customizable_name 
        ) 
        values ( 
            p_clip_id, 
            p_url_index, 
            p_param_name, 
            l_param_value,
            p_param_usage, 
            p_encrypted, 
            p_customizable, 
            p_customizable_name 
        );

    end insert_default_url_param; 
  
   /**
    * Private Helper function that decomposes the lookup_clip. This does not
    * have any dependencies on the Portal user id.
    */  
    procedure lookup_clip_helper(
        p_clip_id               in  integer,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_obfuscation_key       in  varchar2
    )
    is
    begin
        -- Second REF CURSOR that will have as many rows as there are urls.
        open p_cv_browse_urls for

            -- Reuse the b.password as a NULL identifier so that it does not
            -- get decrypted when it's NULL.
            select b.url_index, b.url, b.method, b.req_enc,
                b.auth_type, b.auth_realm, b.username, 
                decrypt_if_needed(
                    WCR_TRUE_L,
                    p_obfuscation_key,
                    b.password
                )
            from wwwcp_browse_url$ b 
            where b.clip_id = p_clip_id
            order by b.url_index;
         
        -- Third REF CURSOR that will have as many rows as there are params.
        open p_cv_default_url_params for

            select d.url_index, d.param_name, 
                decrypt_if_needed(
                    d.encrypted, 
                    p_obfuscation_key, 
                    d.param_value
                ),
                d.param_usage, d.encrypted, d.customizable, d.customizable_name
            from wwwcp_default_url_parameter$ d
            where d.clip_id = p_clip_id;
       
    end lookup_clip_helper;

   /**
    * This method first resolves the user to use when looking up the locator 
    * table, then fills up the two out-parameters path and signature with its
    * lookup results from the locator table.
    *
    * The logic behind the resolving of the user is basically, if the user
    * exists in the locator table then he has his own fuzzy match updated path
    * and signature. Otherwise, the lookup with use the default. 
    *
    * The logic behind the storage is that by default, during an insertion of a
    * clip, an insertion into the locator table will happen with the default
    * user WCR_DEFAULT_USER for Phase II and "edit default" of Phase I. In
    * addition, in Phase I, when a customize happens where a new insertion is
    * done a new entry is stored in the locator with the current portal user as
    * well. Finally, when a fuzzy match happens for a particular user in a 
    * specific customization context (identified usually by the four keys), an
    * entry is again stored into the locator with the aforementioned four keys.
    * This logic applies the same for Phase I and Phase II.
    */
    procedure lookup_clip_locator(
        p_provider_id         in     varchar2, 
        p_portlet_id          in     varchar2,
        p_portlet_instance_id in     varchar2, 
        p_portal_user_id      in     varchar2,
        p_clip_path              out varchar2,
        p_clip_signature         out varchar2
    )
    is
        l_cust_id        integer;
        l_user_is_owner  integer;
        l_portal_user_id varchar2(512);
    begin
       
        -- Don't create the entry if it does not exist
        l_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => FALSE
        );
        
        -- Didn't find anything, set everything to NULL and return
        if l_cust_id is NULL then
            p_clip_path := NULL;
            p_clip_signature := NULL;
            return;
        end if;
        
        -- After resolving the user to be used by the locator query above,
        -- lookup the locator table for the path and signature of the clipping
        -- definition.
        select l.clip_path, l.clip_signature
        into p_clip_path, p_clip_signature
        from wwwcp_clipping_locator$ l
        where l.cust_id = l_cust_id;
        
    exception
       
        -- Default to NULL and have the caller inspect it
        when NO_DATA_FOUND then
            p_clip_path := NULL;
            p_clip_signature := NULL;
       
    end lookup_clip_locator;

   /**
    * It looks up a Clip with the right mapping, and returns the whole
    * of the clipping definition in the form of a list of cursors.
    *
    * It returns the clip id after the lookup is done to allow for later 
    * processing using the clip id.
    */
    procedure lookup_clip(
        p_provider_id           in  varchar2, 
        p_portlet_id            in  varchar2,
        p_portlet_instance_id   in  varchar2, 
        p_portal_user_id        in  varchar2,
        p_cv_clipping           out clipping_cursor,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_obfuscation_key       in  varchar2
    ) is
        l_clip_path       varchar2(1024);
        l_clip_signature  varchar2(4000);
        l_query_clip_id   integer;
        l_current_cust_id integer;
        l_owner_cust_id   integer;
    begin

        -- Find the clip id that the four keys map to. The implementation of
        -- the lookup_clip_id_with_def will take into account the differences
        -- between Phase I and Phase II.
        l_query_clip_id := lookup_clip_id_with_def (
            p_provider_id, 
            p_portlet_id, 
            p_portlet_instance_id, 
            p_portal_user_id
        );
        
        -- Lookup the locator (Method already catches the NO_DATA_FOUND)
        lookup_clip_locator(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_clip_path           => l_clip_path,
            p_clip_signature      => l_clip_signature
        );
        
        -- Find the customization id based on the mapping
        if p_portlet_id = WCR_GENERIC_PORTLET then
          select cm.cust_id 
          into l_owner_cust_id
          from wwwcp_customization_map$ cm
          where cm.clip_id = l_query_clip_id;
        else 
          l_owner_cust_id := -1;
        end if;
        
        -- Find the current customization
        l_current_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => FALSE
        );
        
        -- First REF CURSOR that will have one row.
        open p_cv_clipping for
   
            select l_query_clip_id,
                -- If we don't find one in the locator table then default
                -- to the clipping table.
                case when l_clip_path is NULL then c.clip_path 
                else l_clip_path
                end,
                  
                -- If we don't find one in the locator table then default
                -- to the clipping table.
                case when l_clip_signature is NULL then c.clip_signature 
                else l_clip_signature
                end,
                  
                c.clip_description, c.clip_title, c.clip_timeout, 
                c.clip_max_tries, c.clip_expiration, c.clip_rebindable, 
                  
                -- If the owner is the current user then the creator is the
                -- passed in p_portal_user_id, otherwise, it's WCR_DEFAULT_USER
                case when l_owner_cust_id = l_current_cust_id then 
                    p_portal_user_id
                else 
                    WCR_DEFAULT_USER
                end
                  
            from wwwcp_clipping$ c
            where c.clip_id = l_query_clip_id;
                
        -- Common code sharing that does not depend on the portal user.
        lookup_clip_helper(
            p_clip_id               => l_query_clip_id,
            p_cv_browse_urls        => p_cv_browse_urls,
            p_cv_default_url_params => p_cv_default_url_params,
            p_obfuscation_key       => p_obfuscation_key
        );

    end lookup_clip;
  
   /**
    * This API looks up a Clip directly with the clipping id, and returns the
    * whole of the clipping definition in the form of a list of cursors. This
    * particular API does not return the list of customizations because it does
    * not allow passing in of any portal user id. This is because at the time
    * of lookup, the portal user id is not known to us yet. 
    * 
    * This method is only called by the "static" lookup during provider / 
    * portlet definition init in Phase II. It calls the other Phase II method
    * above with static values of the default provider id, portlet instance id,
    * and portal user id.
    */
    procedure lookup_clip(
        p_clip_id               in  integer,
        p_cv_clipping           out clipping_cursor,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_obfuscation_key       in  varchar2
    )
    is
        l_clip_id integer;
    begin
        -- Forward the call to the Phase II method above with a 
        -- set of values for the foursome that are dummy values because
        -- the lookup will only see the portlet id as the clip id.
        lookup_clip(
            p_provider_id           => NULL,
            p_portlet_id            => to_char(p_clip_id),
            p_portlet_instance_id   => NULL,
            p_portal_user_id        => NULL,
            p_cv_clipping           => p_cv_clipping,
            p_cv_browse_urls        => p_cv_browse_urls,
            p_cv_default_url_params => p_cv_default_url_params,
            p_obfuscation_key       => p_obfuscation_key
        );

    end lookup_clip;

   /**
    * This API will only be called with the Phase I implementation of the
    * Generic Web Clipping Portlet
    *
    * This API will only be called by the Phase II implementation of the
    * Provider Builder creation of Web Clipping type Portlets. It does not have
    * the cursor for Url Parameters because formal Portlet parameter values are
    * stored locally to Portal and passed to us during Show Mode.
    *
    * It is also called by the PL/SQL method that is called by the Phase I
    * implementation of the middle tier for re-use purposes. If the Portlet Id
    * is not the generic Portlet Id, then a call to lookup_clip_id_with_def
    * will be made as it already differentiated itself as Phase I.
    *
    * This API looks up a Clip with the right mapping, and returns the whole
    * of the clipping definition in the form of a list of cursors.
    */
    procedure lookup_clip(
        p_provider_id           in  varchar2, 
        p_portlet_id            in  varchar2,
        p_portlet_instance_id   in  varchar2, 
        p_portal_user_id        in  varchar2,
        p_cv_clipping           out clipping_cursor,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_cv_url_params         out url_param_cursor,
        p_obfuscation_key       in  varchar2
    )
    is
        l_cust_id integer;
    begin

        -- Forward the call to the Phase II implementation method to fill up
        -- the cursors other than the Url Parameters.
        lookup_clip(
            p_provider_id           => p_provider_id,
            p_portlet_id            => p_portlet_id,
            p_portlet_instance_id   => p_portlet_instance_id,
            p_portal_user_id        => p_portal_user_id,
            p_cv_clipping           => p_cv_clipping,
            p_cv_browse_urls        => p_cv_browse_urls,
            p_cv_default_url_params => p_cv_default_url_params,
            p_obfuscation_key       => p_obfuscation_key
        );
        
        l_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => FALSE
        );
        
        -- Last REF CURSOR with 1 row for each customizable parameter.
        open p_cv_url_params for 

            select u.url_index, u.param_name, u.param_value
            from wwwcp_url_parameter$ u
            where u.cust_id = l_cust_id;
    
    end lookup_clip;
  
   /** 
    * This API updates a default url parameter when the Page Designer alters 
    * the behavior of it, either to make it customizable, or to alter the
    * default value of the parameter.
    */
    procedure update_default_url_parameter(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_customizable        in varchar2,
        p_customizable_name   in varchar2,
        p_url_index           in integer,
        p_param_name          in varchar2,
        p_param_value         in varchar2,
        p_obfuscation_key     in varchar2
    )
    is
        l_query_clip_id integer;
        l_param_value   varchar2(2048);
    begin
        -- Find the clip id that will set this whole thing off.
        -- TODO: Find out why this is without def (of course this is done
        -- only in edit default where the portal_user_id is - anyway.
        l_query_clip_id := lookup_clip_id_without_def(
            p_provider_id, 
            p_portlet_id, 
            p_portlet_instance_id, 
            p_portal_user_id
        );
     
        -- Do the updating.
        update wwwcp_default_url_parameter$ u
        set u.customizable = p_customizable, 
           u.customizable_name = p_customizable_name,
           u.param_value = encrypt_if_needed(
               u.encrypted, 
               p_obfuscation_key, 
               p_param_value
           )
        where 
            u.clip_id    = l_query_clip_id and
            u.url_index  = p_url_index     and
            u.param_name = p_param_name;

    end update_default_url_parameter;
    
   /**
    * This API updates the locator of the clipping for a particular user, using
    * the four keys to look up the clip id and then updating the locator table
    * using that clip id.
    *
    * This happens when fuzzy match kicks in and creates a new path
    * and signature for the current user.
    *
    * The four keys are used to uniquely identify a locator pair of path and 
    * signature. The method simply inserts and updates when an exception of
    * duplicate is found. The logic of why the insertion is preferred over the
    * updating is explained in the code.
    */
    procedure update_clip_locator(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2
    )
    is
        l_cust_id integer;
    begin
       
        -- Lookup the customization if it exists; we are in an insert action
        -- so we do want to insert a new one if it did not exist.
        l_cust_id := lookup_customization(
            p_provider_id         => p_provider_id,
            p_portlet_id          => p_portlet_id,
            p_portlet_instance_id => p_portlet_instance_id,
            p_portal_user_id      => p_portal_user_id,
            p_create              => TRUE
        );
        
        -- Do the inserting because when this function is called; chances are
        -- there is still no entries in the table for this user. When an insert
        -- has been done, chances are, updates will be less frequent because
        -- the user has his own private clip_signature already which is less
        -- likely to change.
        insert into wwwcp_clipping_locator$ l (
            cust_id,
            clip_path,
            clip_signature
        )
        values (
            l_cust_id,
            p_clip_path,
            p_clip_signature
        );
     
    exception

        -- The insert is unsuccessful because it already exists.
        -- Theoretically this happens less often because once an entry exists
        -- for a particular user via the INSERT, it is less likely that another
        -- fuzzy match will happen for the same user.
        when DUP_VAL_ON_INDEX then
        update wwwcp_clipping_locator$ l
        set l.clip_path = p_clip_path, 
            l.clip_signature = p_clip_signature
        where l.cust_id = l_cust_id;
    
    end update_clip_locator;

   /**
    * Update the high level properties of the clipping
    */
    procedure update_clip_properties(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    )
    is
        l_query_clip_id integer;
    begin
        -- Find the clip id that will set this whole thing off.
        -- Want to find the id WITH default because we only need to modify
        -- our private view storage of the signature and path.
        l_query_clip_id := lookup_clip_id_without_def(
            p_provider_id,
            p_portlet_id,
            p_portlet_instance_id, 
            p_portal_user_id
        );
     
        -- Update the clipping table with the new information
        update wwwcp_clipping$ c
        set c.clip_description = p_clip_description,
            c.clip_title = p_clip_title,
            c.clip_timeout = p_clip_timeout,
            c.clip_max_tries = p_clip_max_tries,
            c.clip_expiration = p_clip_expiration,
            c.clip_rebindable = p_clip_rebindable
        where c.clip_id = l_query_clip_id;
    end update_clip_properties;
  

   /**
    * This API removes all customization for a particular clip for a particular
    * portal user id.
    */
    procedure remove_clip_customization(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    ) 
    is
        l_query_cust_id integer;
    begin

        -- For phase II, remove all the customization for this clip
        if p_portlet_id <> WCR_GENERIC_PORTLET then
            
          -- remove the customization paramters
          delete from wwwcp_url_parameter$ u
          where u.cust_id in (
            select m.cust_id
            from wwwcp_customization_map$ m
            where
              m.clip_id = to_number(p_portlet_id)
            );
          
          -- remove the location table due to fuzzy match
          delete from wwwcp_clipping_locator$ l
            where l.cust_id in (
                select m.cust_id
                from wwwcp_customization_map$ m
                where m.clip_id = to_number(p_portlet_id)
            );

        -- Phase I                  
        else 
  
          -- If the user is the default user, then we'll need to clean up
          -- more than his own cust id, we'll need to clean up all those
          -- other cust ids who bring to the same clip.
          --
          -- The Set from which the list of cust ids we want is those that not
          -- only satisfies the criteria of being in the same portlet instance
          -- but also of having a map that was pointing to NULL indicating
          -- that it did not have its own clip
          if p_portal_user_id = WCR_DEFAULT_USER then
           
            -- TODO: Create temporary table for the "IN"
            -- Delete from the customization table.
            delete from wwwcp_url_parameter$ u
            where u.cust_id in (
                select cust.cust_id
                from wwwcp_customization$ cust, wwwcp_customization_map$ m
                where
                    cust.provider_id         = p_provider_id and
                    cust.portlet_id          = p_portlet_id and
                    cust.portlet_instance_id = p_portlet_instance_id and
                    cust.cust_id             = m.cust_id and
                    m.clip_id                is NULL
            );
            
            -- From the locator table as well
            delete from wwwcp_clipping_locator$ l
            where l.cust_id in (
                select cust.cust_id
                from wwwcp_customization$ cust, wwwcp_customization_map$ m
                where
                    cust.provider_id         = p_provider_id and
                    cust.portlet_id          = p_portlet_id and
                    cust.portlet_instance_id = p_portlet_instance_id and
                    cust.cust_id             = m.cust_id and
                    m.clip_id                is NULL
            );
        
          else
           
            -- Find the clip id with the default user
            l_query_cust_id := lookup_customization(
                p_provider_id, 
                p_portlet_id, 
                p_portlet_instance_id, 
                p_portal_user_id,
                FALSE
            );
        
            -- Found customizations to kill
            if (l_query_cust_id is not NULL) then
           
                -- Delete from the customization table.
                delete from wwwcp_url_parameter$ u
                where u.cust_id = l_query_cust_id;
           
                -- From the locator table as well
                delete from wwwcp_clipping_locator$ l
                where l.cust_id = l_query_cust_id;
            end if;
        
          end if;
          
        end if;
       
    end remove_clip_customization;    
  
   /**
    * This API removes all the clipping definitions associated with a given
    * Portlet instance in Phase I.
    */
    procedure remove_all_clips_for_instance(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2
    )
    is
    begin

        -- Delete from the clipping table cascade with same clipid.
        delete from wwwcp_clipping$ c 
        where c.clip_id in (
            select m.clip_id 
            from wwwcp_customization_map$ m
            where m.cust_id in (
                select cust.cust_id
                from wwwcp_customization$ cust
                where 
                    cust.provider_id         = p_provider_id and
                    cust.portlet_id          = p_portlet_id and
                    cust.portlet_instance_id = p_portlet_instance_id
            )
        );

        -- Delete from the customization map table cascade with same cust id
        delete from wwwcp_customization_map$ m
        where m.cust_id in (
            select cust.cust_id
            from wwwcp_customization$ cust
            where 
                cust.provider_id         = p_provider_id and
                cust.portlet_id          = p_portlet_id and
                cust.portlet_instance_id = p_portlet_instance_id
        );            
        
    end remove_all_clips_for_instance;

    /** 
     * Phase II specific to remove a portlet definition from provider builder.
     */
    procedure remove_definition(
        p_clip_id          in integer
    )
    is
    begin
        delete from wwwcp_clipping$ c where c.clip_id = p_clip_id;
        delete from wwwcp_customization_map$ m
            where m.cust_id in (
                select cust.cust_id
                from wwwcp_customization$ cust 
                where cust.portlet_id = to_char(p_clip_id)
            ); 
    end remove_definition;      

    /** 
     * Phase II specific to remove a instance from a page.
     */
    procedure remove_instance(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    )
    is      
    begin
        delete from wwwcp_customization_map$ m
            where m.cust_id in (
                select cust.cust_id
                from wwwcp_customization$ cust
                where 
                    cust.provider_id         = p_provider_id and
                    cust.portlet_id          = p_portlet_id and
                    cust.portlet_instance_id = p_portlet_instance_id and 
                    cust.portal_user_id      = p_portal_user_id
            );
        
    end remove_instance;

   /**
    * This API encrypts p_value based on the boolean value of p_encrypted.
    */
    function encrypt_if_needed(
        p_encrypted       in varchar2,
        p_obfuscation_key in varchar2, 
        p_value           in varchar2
    ) return varchar2 is
        l_value varchar2(2048);
    begin
        if p_encrypted is NULL or p_value is NULL then
            return p_value;
        end if;

        if p_encrypted = WCR_TRUE_U or p_encrypted = WCR_TRUE_L then 
            dbms_obfuscation_toolkit.des3encrypt(
                input_string     => p_value,
                key_string       => p_obfuscation_key,
                encrypted_string => l_value
            );
            return l_value;
        else
            return p_value;
        end if;

    end encrypt_if_needed;
  
   /**
    * This API decrypts p_value based on the boolean value of p_encrypted.
    */
    function decrypt_if_needed(
        p_encrypted       in varchar2,
        p_obfuscation_key in varchar2, 
        p_value           in varchar2
    ) return varchar2 is
        l_value varchar2(2048);
    begin
        if p_encrypted is NULL or p_value is NULL then
            return p_value;
        end if;

        if p_encrypted = WCR_TRUE_U or p_encrypted = WCR_TRUE_L then
            dbms_obfuscation_toolkit.DES3Decrypt(
                input_string     => p_value,
                key_string       => p_obfuscation_key,
                decrypted_string => l_value
            );
            return l_value;
        else
            return p_value;
        end if;

    end decrypt_if_needed;
  
   /**
    * This API inserts a new row of element name / path into the element 
    * paths table, given a specific clip id. 
    *
    * This denotes that for the given clipping there exists a scraping 
    * definition that is identified by all rows having that clip's clip id.
    */
    procedure insert_element_path(
        p_clip_id              in integer,
        p_clip_element_name    in varchar2,
        p_clip_element_path    in varchar2
    )      
    is 
    begin
        insert into wwwcp_element_path$ (
            clip_id,
            element_name, 
            element_path
        )
        values (
            p_clip_id,
            p_clip_element_name,
            p_clip_element_path
        );
    end insert_element_path;

   /**
    * This API inserts a new row of a render event given a specific clip id.
    *
    * It uses the clip id to look up information from the clip definition
    * tables to fill up this row.
    */
    procedure insert_render_event(
        p_clip_id           in integer,
        p_render_status     integer,
        p_render_type       integer,
        p_render_start      date,
        p_render_end        date,
        p_fuzzy_used        integer,
        p_db_lookup         integer,
        p_http_latency      integer,
        p_render_user       varchar2,
        p_error_cause       varchar2,
        p_event_description varchar2,
        p_clip_input        varchar2,
        p_clip_output       varchar2
    )      
    is 
        l_clip_description varchar2(1024);
        l_clip_title       varchar2(512);
        l_clip_timeout     integer;
        l_effective_url    varchar2(2048);
    begin
        -- First get the information from the clipping table
        select c.clip_description, c.clip_title, c.clip_timeout
        into l_clip_description, l_clip_title, l_clip_timeout
        from wwwcp_clipping$ c
        where c.clip_id = p_clip_id;

        -- Then ask for the last url from the browse url table
        select b.url 
        into l_effective_url
        from wwwcp_browse_url$ b
        where b.clip_id = p_clip_id and
            b.url_index = (
                select max(b2.url_index) 
                from wwwcp_browse_url$ b2
                where b2.clip_id = p_clip_id
            );
        
        insert into wwwcp_render_log$ (
            clip_id           ,
            clip_description  ,
            clip_title        ,
            clip_timeout      ,
            effective_url     ,
            render_status     ,
            render_type       ,
            render_start      ,
            render_end        ,
            fuzzy_used        ,
            db_lookup         ,
            http_latency      ,
            render_user       ,
            error_cause       ,
            event_description ,
            clip_input        ,
            clip_output
        )
        values (
            p_clip_id           ,
            l_clip_description  ,
            l_clip_title        ,
            l_clip_timeout      ,
            l_effective_url     ,
            p_render_status     ,
            p_render_type       ,
            p_render_start      ,
            p_render_end        ,
            p_fuzzy_used        ,
            p_db_lookup         ,
            p_http_latency      ,
            p_render_user       ,
            p_error_cause       ,
            p_event_description ,
            p_clip_input        ,
            p_clip_output       
        );

    end insert_render_event;

   /**
    * This API looks up rows of element name / path from the element 
    * paths table, given a specific clip id. 
    *
    * This denotes that for the given clipping there exists a scraping 
    * definition that is identified by all rows having that clip's clip id.
    */
    procedure lookup_element_paths(
        p_clip_id              in integer,
        p_cv_element_paths     out clip_element_path_cursor
    )
    is 
    begin
        open p_cv_element_paths for
            select ep.element_name, ep.element_path
            from wwwcp_element_path$ ep
            where ep.clip_id = p_clip_id;
    end lookup_element_paths;
    
   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by a specific clip id.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    */
    procedure lookup_render_events(
        p_clip_id          in integer,
        p_cv_render_events out render_event_cursor
    )
    is 
    begin
        open p_cv_render_events for
            select 
                re.clip_id, re.clip_description, re.clip_title, 
                re.clip_timeout, re.effective_url, 
                re.render_status, re.render_type, 
                re.render_start, re.render_end,
                re.fuzzy_used, re.db_lookup, re.http_latency, re.render_user,
                re.error_cause, re.event_description, 
                re.clip_input, re.clip_output
            from wwwcp_render_log$ re
            where re.clip_id = p_clip_id;
    end lookup_render_events;

   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by a specific effective url.
    *
    * This is useful for business queries that center on not the clip id but by
    * the actual location of where the clips are found. For example, if Yahoo
    * can do a search of all the events that went to itself, it can find out
    * how to charge each user for what they have received.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    */
    procedure lookup_render_events(
        p_effective_url    in  varchar2,
        p_cv_render_events out render_event_cursor
    )
    is 
    begin
        open p_cv_render_events for
            select 
                re.clip_id, re.clip_description, re.clip_title, 
                re.clip_timeout, re.effective_url, 
                re.render_status, re.render_type, 
                re.render_start, re.render_end,
                re.fuzzy_used, re.db_lookup, re.http_latency, re.render_user,
                re.error_cause, re.event_description, 
                re.clip_input, re.clip_output
            from wwwcp_render_log$ re
            where re.effective_url like p_effective_url || '%';
    end lookup_render_events;
    
   /**
    * This method returns the domain of a URL for lookup purposes.
    */
    function get_domain(p_url in varchar2) return varchar2 is
        l_url               varchar2(2048);
        l_host              varchar2(32);
        l_without_protocol  number;
        l_first_slash_index number;
        l_first_dot_index   number;
    begin
        -- Don't do anything with NULL
        if p_url is NULL then
            return p_url;
        end if;
        
        -- After the protocol is where we want the string to start.
        l_without_protocol := instr(p_url, '://', 1, 1);
        
        -- If we did indeed have a protocol, then we need to rewrite the p_url.
        if l_without_protocol > 0 then
            l_url := substr(p_url, l_without_protocol + length('://'));
        else
            l_url := p_url;
        end if;
        
        -- First get the first index of '/'
        l_first_slash_index := instr(l_url, '/', 1, 1);
        
        -- If the slash is not found, then return the whole URL
        if l_first_slash_index = 0 then
            -- Host name is the url
            l_host := l_url;
        else
            -- Host name is the substring before the slash
            l_host := substr(l_url, 1, l_first_slash_index - 1);
        end if;
        
        -- Find the first '.' within host
        l_first_dot_index := instr(l_host, '.', 1, 1);
        
        -- Get the domain from the host
        return substr(l_host, l_first_dot_index + 1);
    
    exception
        -- Catch all exception handling
        when others then
            return p_url;
    end;

   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by the domain.
    *
    * This is useful for business queries that center on not the clip id but by
    * the domain of where the clips are found. For example, if Yahoo can do a
    * search of all the events that went to *.yahoo.com, it can find out how to
    * charge each user for what they have rendered.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    */
    procedure lookup_render_events(
        p_effective_domain in  varchar2,
        p_cv_render_events out render_event_cursor
    )
    is 
        l_domain varchar2(32);
    begin
          -- If we begin with a ., remove it
          if instr(p_effective_domain, '.', 1, 1) = 1 then
              l_domain := substr(p_effective_domain, 2);
          else
              l_domain := p_effective_domain;
          end if;
          
          open p_cv_render_events for
            select 
                re.clip_id, re.clip_description, re.clip_title, 
                re.clip_timeout, re.effective_url, 
                re.render_status, re.render_type, 
                re.render_start, re.render_end,
                re.fuzzy_used, re.db_lookup, re.http_latency, re.render_user,
                re.error_cause, re.event_description, 
                re.clip_input, re.clip_output
            from wwwcp_render_log$ re
            where get_domain(re.effective_url) = l_domain;
    end lookup_render_events;
    
   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by the user that requested the render.
    *
    * This is useful for business queries that center on not the clip id but by
    * the user to visualize how much a user has rendered, regardless of where
    * the clipping is from, so it can charge for a flat fee based on for 
    * example, the number of render events for a particular user.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    *
    * @param p_render_user      the user on behalf of whom the render request
    *                           is made.
    * @param p_cv_render_events the list of render events that are associated
    *                           with this clip
    */
    procedure lookup_render_events(
        p_render_user      in  varchar2,
        p_cv_render_events out render_event_cursor
    )
    is 
    begin
        open p_cv_render_events for
            select 
                re.clip_id, re.clip_description, re.clip_title, 
                re.clip_timeout, re.effective_url, 
                re.render_status, re.render_type, 
                re.render_start, re.render_end,
                re.fuzzy_used, re.db_lookup, re.http_latency, re.render_user,
                re.error_cause, re.event_description, 
                re.clip_input, re.clip_output
            from wwwcp_render_log$ re
            where re.render_user = p_render_user;
    end lookup_render_events;
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table.
    *
    * This is useful as a cleansing measure that an organization can do between
    * the different phases of rolling out their Web Clipping usage initiative.
    * It can be used to reset the stage before going into development testing
    * phase, or when the rollout is going live and the company wishes to begin
    * charging its subscribers for their usages, or another possible cleansing
    * may occur at for example, every month end or year end when the data
    * recorded is no longer useful.
    */
    procedure remove_all_render_events
    is 
    begin
        delete from wwwcp_render_log$;
    end remove_all_render_events;
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table for
    * a given rendering user.
    *
    * This is useful as a cleansing measure that an organization can do for
    * example, labeling the development user to be some constant user id and
    * then removing all rendering events when it goes live.
    */
    procedure remove_render_events(p_render_user in varchar2)
    is
    begin
        delete from wwwcp_render_log$ re
        where re.render_user = p_render_user;
    end remove_render_events;
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table for
    * a given starting or stopping time before or after the query time.
    *
    * This is useful as a cleansing measure that an organization can do for
    * example, all rendering events that started or stopped before a certain 
    * date are deemed development efforts and therefore should be disregarded.
    */
    procedure remove_render_events(
        p_query_time   in date,
        p_query_before in number,
        p_query_start  in number
    )
    is
    begin
        -- Remove all those with some time before query time
        if p_query_before = 1 then
            if p_query_start = 1 then
                -- Remove all those with start time before query time
                delete from wwwcp_render_log$ re
                where re.render_start < p_query_time;
            else
                -- Remove all those with stop time before query time
                delete from wwwcp_render_log$ re
                where re.render_end < p_query_time;
            end if;
        -- Remove all those with some time after query time
        else 
            if p_query_start = 1 then
                -- Remove all those with start time after query time
                delete from wwwcp_render_log$ re
                where re.render_start > p_query_time;
            else
                -- Remove all those with stop time after query time
                delete from wwwcp_render_log$ re
                where re.render_end > p_query_time;
            end if;
        end if;
    end remove_render_events;
   
end wwwcp_api_registry;
/

show errors package body wwwcp_api_registry

