/* @Copyright (C) 2002, Oracle Corporation. All rights reserved. */

create or replace package wwwcp_api_registry is
  
   /**
    * @headcom
    *
    * Web Clipping Provider Registry APIs providing storage for Web Clipping
    * Definitions.
    * 
    * This Package provides APIs to allow the Web Clipping Web Provider to 
    * query, insert, delete, and update the clipping definitions that it uses 
    * as content for the Web Clipping Portlet.
    *
    * @scope Internal
    */

   /** 
    * This is the provider id that is used when none is given to us to use
    * when inserting from the Provider Builder.
    */
    WCR_DEFAULT_PROVIDER constant varchar2(24) := '-';

   /** 
    * This is the portlet id that is used for the generic portlet id (Phase I).
    */
    WCR_GENERIC_PORTLET constant varchar2(24) := '1';

   /** 
    * This is the portlet instance name that is used when none is given to us
    * to use when inserting from the Provider Builder.
    */
    WCR_DEFAULT_PORTLET_INSTANCE constant varchar2(24) := '-';

   /** 
    * This is the username that is used as a Portal User Id when the clipping
    * has been created using "Edit Defaults".
    */
    WCR_DEFAULT_USER constant varchar2(24) := '-';

   /**
    * This describes a record type used to return a single high level Clipping
    * Definition.
    *
    * This structure is used by the lookup APIs to encapsulate the information
    * that is retrieved from the clipping$ and clipping_locator$ tables. It is
    * used to describe the cursor that will be returned as an OUT parameter of
    * lookup_clip.
    *
    * @field clip_id          clip identifier
    * @field clip_path        path in HTML within the Web Page that allows the
    *                         provider to locate the actual clipping
    * @field clip_signature   a checksum / signature of the clip that allows 
    *                         for a check to happen when displaying the clip to
    *                         make sure that the correct clip is found
    * @field clip_description textual description of the clip created
    * @field clip_title       title of the clip that will be used to double as
    *                         the title of the Portlet
    * @field clip_timeout     timeout in seconds that allows the Portlet to be
    *                         timed out in the event that an HTTP connection
    *                         either could not be open or is too slow
    * @field clip_max_tries   number of times to try when timeout happens
    * @field clip_expiration  the expiration time of this particular clip in 
    *                         minutes. This not only stores the user specified
    *                         expiration value, but also the kind of expiration
    *                         source he desires. The storage is agnostic to the
    *                         logic of deciding which is which.
    * @field clip_rebindable  indicates whether or not this clipping
    *                         definition can be changed
    * @field clip_creator     the creator of the clipping definition
    */
    type rec_clipping is record (
        clip_id          integer,
        clip_path        varchar2(1024),
        clip_signature   varchar2(4000),
        clip_description varchar2(1024),
        clip_title       varchar2(512),
        clip_timeout     integer,
        clip_max_tries   integer,
        clip_expiration  integer,
        clip_rebindable  varchar2(1),
        clip_creator     varchar2(512));
  
   /**
    * This describes a record type used to return a single browse url 
    * that's part of a clip.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_browse_url$ table. 
    * It is used to describe the cursor that will be returned as an 
    * OUT parameter of lookup_clip.
    *
    * @field url_index  index of the URL within the clipping definition 
    *                   as the URLs need to be ordered
    * @field url        URL that was browsed to reach this clip
    * @field method     Method used to access the URL (GET, POST)
    * @field req_enc    The Java encoding for the request parameters
    * @field auth_type  HTTP Authentication type (Basic, Digest, and so on)
    * @field auth_realm HTTP Authentication realm
    * @field username   If needed for authentication to the URL
    * @field password   If needed for authentication to the URL
    */
    type rec_browse_url is record (
        url_index  integer,
        url        varchar2(2048),
        method     varchar2(8),
        req_enc    varchar2(16),
        auth_type  varchar2(8),
        auth_realm varchar2(256),
        username   varchar2(128),
        password   varchar2(128));
  
   /**
    * This describes a record type used to return a single default 
    * parameter for a particular browse url that's part of a clip.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_default_url_parameter$ 
    * table. It is used to describe the cursor that will be returned 
    * as an OUT parameter of lookup_clip.
    *
    * @field url_index         index of the URL within the clipping 
    *                          definition as the URLs need to be ordered
    * @field param_name        name of the HTTP parameter
    * @field param_value       value of the HTTP parameter
    * @field param_usage       The way in which this HTTP parameter is to be 
    *                          used including request parameters or headers
    * @field encrypted         indicates whether this parameter is encrypted
    * @field customizable      indicates whether this parameter is open for
    *                          Page Viewers to set their own values
    * @field customizable_name If the parameter is customizable, then this
    *                          denotes the name the Page Viewer will see when
    *                          trying to set his own value
    */
    type rec_def_url_param is record (
        url_index         integer,
        param_name        varchar2(128),
        param_value       varchar2(128),
        param_usage       varchar2(8),
        encrypted         varchar2(1),
        customizable      varchar2(1),
        customizable_name varchar2(128));
  
   /**
    * This describes a record type used to return a single customized 
    * parameter for a particular browse url that's part of a clip.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_url_parameter$ 
    * table. It is used to describe the cursor that will be returned 
    * as an OUT parameter of lookup_clip.
    *
    * @field url_index   index of the URL within the clipping definition 
    *                    as the URLs need to be ordered
    * @field param_name  name of the HTTP parameter
    * @field param_value value of the HTTP parameter
    */
    type rec_url_param is record (
        url_index   integer,
        param_name  varchar2(128),
        param_value varchar2(128));

   /**
    * This describes a record type used to return a single clip element 
    * path as it represents part of the scraping definition of a clipping.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_element_path$ table. 
    * It is used to describe the cursor that will be returned as an OUT 
    * parameter of lookup_element_paths.
    *
    * @field element_name name of the element usually corresponding to
    *                     an output parameter name
    * @field element_path path to reach the said output element within
    *                     the clip confines
    */
    type rec_clip_element_path is record (
        element_name  varchar2(128),
        element_path  varchar2(256));

   /**
    * This describes a record type used to return a single clipping rendering
    * event.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_render_log$ table. 
    * It is used to describe the cursor that will be returned as an OUT 
    * parameter of lookup_render_events.
    *
    * @field clip_id           the clip id that allows the fetching of the 
    *                          other facets of the clipping definition to 
    *                          populate the events table
    * @field clip_description  textual description of the clip rendered
    * @field clip_title        title of the clip that was rendered
    * @field clip_timeout      timeout in milliseconds that allows the clipping
    *                          that was rendered to be timed out. This could be
    *                          an indication of the quality of service.
    * @field effective_url     the url where the clip resides. This is usually
    *                          the last url declared in the clipping definition
    *                          clipping definition, where the clip would reside
    * @field render_status     a number that indicates either success of 
    *                          failure of the rendering call.
    * @field render_type       tells what type of rendering is in question,
    *                          whether it be for Portal Show Mode, for 
    *                          Wireless default SimpleResult show mode, or 
    *                          for the Wireless connector show mode
    * @field render_start      starting time of the rendering
    * @field render_end        ending time of the rendering
    * @field fuzzy_used        indicates whether fuzzy match was kicked in
    * @field db_lookup         the time it takes in milliseconds to look up
    *                          the clipping definition from the database
    * @field http_latency      the time it takes to reach the first byte to
    *                          read from the http remote site
    * @field render_user       the user (together with a possible realm) that
    *                          the rendering was done for
    * @field error_cause       the cause of the error if the status indicated
    *                          a failure of the rendering
    * @field event_description this fieldeter provides more information about
    *                          the logged event
    * @field clip_input        the input provided to render this clipping. It
    *                          is usually provided in the form of an HTTP URL
    *                          query string like abc=def
    * @field clip_output       the possible output (only if it's small) of 
    *                          rendering, if it gives any more hints on the
    *                          billing process.
    */
    type rec_render_event is record (
        clip_id           integer,
        clip_description  varchar2(1024),
        clip_title        varchar2(512),
        clip_timeout      integer,
        effective_url     varchar2(2048),
        render_status     integer,
        render_type       integer,
        render_start      date,
        render_end        date,
        fuzzy_used        integer,
        db_lookup         integer,
        http_latency      integer,
        render_user       varchar2(512),
        error_cause       varchar2(256),
        event_description varchar2(256),
        clip_input        varchar2(128),
        clip_output       varchar2(256));
    
   /**
    * This describes a record type used to return a single clip metadata
    * as it is the kind of information returned when finding clips with
    * certain criteria.
    *
    * This structure is used by the lookup APIs to encapsulate the 
    * information that is retrieved from the wwwcp_element_path$ table. 
    * It is used to describe the cursor that will be returned as an OUT 
    * parameter of lookup_clip_ids_with_contains.
    *
    * @field clip_id          the clip id of the result entry, using which 
    *                         subsequent calls can be made to lookup a whole 
    *                         clipping definition.
    * @field clip_title       this allows the user to graphically display the 
    *                         result of the find for users to easily identify 
    *                         the desired clip
    * @field clip_description this allows the user to graphically display the 
    *                         result of the find for users to easily identify 
    *                         the desired clip
    */
    type rec_clipping_metadata is record (
        clip_id          integer,
        clip_title       varchar2(512),
        clip_description varchar2(1024));

   /**
    * This is the REF Cursor type associated with the clipping record.
    */
    type clipping_cursor          is ref cursor return rec_clipping;

   /**
    * This is the REF Cursor type associated with the browse url record.
    */
    type browse_url_cursor        is ref cursor return rec_browse_url;
  
   /**
    * This is the REF Cursor type associated with the default url param record.
    */
    type def_url_param_cursor     is ref cursor return rec_def_url_param;

   /**
    * This is the REF Cursor type associated with the customized url param
    * record.
    */
    type url_param_cursor         is ref cursor return rec_url_param;

   /**
    * This is the REF Cursor type associated with the element path record
    */
    type clip_element_path_cursor is ref cursor return rec_clip_element_path;
    
   /**
    * This is the REF Cursor type associated with the clipping metadata record
    */
    type clipping_metadata_cursor is ref cursor return rec_clipping_metadata;
    
   /**
    * This is the REF Cursor type associated with the render event record
    */
    type render_event_cursor is ref cursor return rec_render_event;

   /**
    * This API performs a lookup the Clip Id associated with the given
    * provider id, portlet id, portlet instance id and portal user, and will
    * default to use the WCR_DEFAULT_USER which is used for clipping
    * definitions created using "Edit Defaults", when one is not found 
    * associated with the passed in portal user id.
    *
    * It first tries to use the passed in portal user id to find his 
    * personalized web clipping. If nothing is found this API defaults to
    * use the default user specified by WCR_DEFAULT_USER to find the id of the
    * web clipping created by the page designer using "Edit Defaults".
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    *
    * @return                       the id of the clipping whose definition is
    *                               to be displayed by a subsequent call to
    *                               lookup the other tables
    */
    function lookup_clip_id_with_def(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    )
    return integer deterministic;
    
   /**
    * This API returns the list of clip ids that have a title or description 
    * that contains the passed in String criterion.
    *
    * @param p_str_contains the String that is used to do a lookup in the
    *                       clipping table for containment within the title
    *                       or the description.
    */
    function lookup_clip_ids_with_contains(
        p_str_contains in varchar2
    )
    return clipping_metadata_cursor;
    
   /**
    * This API performs a lookup the Clip Id associated with the given
    * provider id, portlet id, portlet instance id and portal user, without
    * defaulting to use the WCR_DEFAULT_USER which is used for clipping
    * definitions created using "Edit Defaults".
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    *
    * @return                       the Clip Id as mapped to the specified
    *                               Portlet Instance that is associated with
    *                               the user identified by p_portal_user_id. If
    *                               no clip id is found for him, this does not
    *                               try to look for one that has been created
    *                               using "Edit Defaults".
    */
    function lookup_clip_id_without_def(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    )
    return integer deterministic;
  
   /**
    * This API inserts an entry into the mapping table to denote that a Portlet
    * has been dropped onto a page.
    *
    * This means that a default mapping has already been setup to point to the
    * clipping table. It works for both Phase I and Phase II in different ways.
    * In Phase I, the instantiation of a Portlet happens before the Clipping
    * Definition has been created. In Phase II, the instantiation of a Portlet
    * happens after the Clipping Definition has been created. This requires a
    * different logical flow of what needs to be
    * inserted into the table.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    */
    function insert_customization(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2
    ) return integer;
    
   /**
    * This API creates an instance of the Portlet as it is born after dropping
    * itself onto a Portal Page. 
    *
    * In essence a default customization is being created so it calls the
    * insert_customization method with WCR_DEFAULT_USER as the user that is
    * doing so.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    */
    procedure create_instance(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    );

   /**
    * This method first resolves the user to use when looking up the locator 
    * table, then fills up the two out-parameters path and signature with its
    * lookup results from the locator table.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults. This parameter
    *                               is updated with the correct resolution of
    *                               the user that was used to do the locator 
    *                               table lookup.
    * @param  p_clip_path           path in HTML within the Web Page that 
    *                               allows the provider to locate the actual
    *                               clipping. 
    * @param  p_clip_signature      a checksum / signature of the clip that 
    *                               allows for a check to happen when 
    *                               displaying the clip to make sure that the
    *                               correct clip is found.
    */
    procedure lookup_clip_locator(
        p_provider_id         in     varchar2, 
        p_portlet_id          in     varchar2,
        p_portlet_instance_id in     varchar2, 
        p_portal_user_id      in     varchar2,
        p_clip_path              out varchar2,
        p_clip_signature         out varchar2
    );

   /**
    * This API will only be called with the Phase I implementation of the
    * Generic Web Clipping Portlet.
    *
    * This API looks up a Clip with the right mapping, and returns the whole
    * of the clipping definition in the form of a list of cursors.
    *
    * @param  p_provider_id           provider Id of the Web Clipping Provider 
    *                                 (Usually constant) as multiple Portal 
    *                                 instances can reuse the same Web Clipping
    *                                 Provider
    * @param  p_portlet_id            id of the Portlet provided by the Web 
    *                                 Clipping Provider, which is constant for
    *                                 now (a value of 1) but for future
    *                                 provisions may be changed as our next
    *                                 phase entails mapping one Clipping
    *                                 Definition to one Portlet Definition
    * @param  p_portlet_instance_id   lowest common denominator for a clipping
    * @param  p_portal_user_id        the Portal user id that this clipping
    *                                 definition is associated with. The frame-
    *                                 work allows each user to have his own 
    *                                 clipping definition for a given Portlet
    *                                 instance. This value can either be the
    *                                 actual Portal user id that is creating
    *                                 the clip or it can be the user identified
    *                                 by WCR_DEFAULT_USER when the clip is
    *                                 being created using edit defaults.
    * @param  p_cv_clipping           Single row cursor that contains 
    *                                 information that is high level for the
    *                                 clip including fields like title and 
    *                                 description
    * @param  p_cv_browse_urls        The list of browse urls for this clipping
    * @param  p_cv_default_url_params The list of default url parameters for
    *                                 this clipping
    * @param  p_cv_url_params         The list of customized url parameters
    * @param  p_obfuscation_key       Obfuscation key passed down from the 
    *                                 caller to decrypt the information that we
    *                                 looked up from tables with encrypted data
    * @return                         the new Clip Id that identifies this 
    *                                 newly inserted Clip
    */
    procedure lookup_clip(
        p_provider_id           in  varchar2, 
        p_portlet_id            in  varchar2,
        p_portlet_instance_id   in  varchar2, 
        p_portal_user_id        in  varchar2,
        p_cv_clipping           out clipping_cursor,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_cv_url_params         out url_param_cursor,
        p_obfuscation_key       in  varchar2
    );

   /**
    * This API looks up a Clip directly with the clipping id, and returns the
    * whole of the clipping definition in the form of a list of cursors. 
    *
    * This particular API does not return the list of customizations because 
    * it does not allow passing in of any portal user id. This is because at 
    * the time of lookup, the portal user id is not known to us yet.
    * 
    * This method is only called by the "static" lookup during provider / 
    * portlet definition init in Phase II. It calls the other Phase II method
    * above with static values of the default provider id, portlet instance id,
    * and portal user id.
    *
    * @param  p_clip_id               clip id as it is stored in the clip
    *                                 definition table
    * @param  p_cv_clipping           Single row cursor that contains 
    *                                 information that is high level for the
    *                                 clip including fields like title and 
    *                                 description
    * @param  p_cv_browse_urls        The list of browse urls for this clipping
    * @param  p_cv_default_url_params The list of default url parameters for
    *                                 this clipping
    * @param  p_obfuscation_key       Obfuscation key passed down from the 
    *                                 caller to decrypt the information that we
    *                                 looked up from tables with encrypted data
    */
    procedure lookup_clip(
        p_clip_id               in  integer,
        p_cv_clipping           out clipping_cursor,
        p_cv_browse_urls        out browse_url_cursor,
        p_cv_default_url_params out def_url_param_cursor,
        p_obfuscation_key       in  varchar2
    );

   /**
    * This API inserts a Clipping definition, mapping it to the four keys that
    * uniquely identify a clipping id, and returns the newly generated Clip Id.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @param  p_clip_path           path in HTML within the Web Page that 
    *                               allows the provider to locate the actual
    *                               clipping
    * @param  p_clip_signature      a checksum / signature of the clip that 
    *                               allows for a check to happen when 
    *                               displaying the clip to make sure that the
    *                               correct clip is found
    * @param  p_clip_description    textual description of the clip created
    * @param  p_clip_title          title of the clip that will be used to
    *                               double as the title of the Portlet
    * @param  p_clip_timeout        timeout in seconds that allows the Portlet
    *                               to be timed out in the event that an HTTP
    *                               connection either could not be open or is
    *                               too slow
    * @param  p_clip_max_tries      number of times to try when timeout happens
    * @param  p_clip_expiration     the expiration time of this particular clip
    *                               in minutes. This not only stores the user
    *                               specified expiration value, but also the
    *                               kind of expiration source he desires. The
    *                               storage is agnostic to the logic of
    *                               deciding which is which.
    * @param  p_clip_rebindable     indicates whether or not this clipping
    *                               definition can be changed
    *
    * @return                       the new Clip Id that identifies this newly
    *                               inserted Clip
    */
    function insert_clip(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    )
    return integer deterministic;
  
   /**
    * This API inserts a Clipping definition in the Phase II implementation. 
    *
    * It happens during the creation of a Portlet using the Web Clipping type.
    * Naturally, we do not have the four keys to put in the mapping table.
    * Thus, we need to simply insert and return the clip id.
    *
    * @param  p_clip_id             an existing Clip Id that identifies this
    *                               clip
    * @param  p_clip_path           path in HTML within the Web Page that 
    *                               allows the provider to locate the actual
    *                               clipping
    * @param  p_clip_signature      a checksum / signature of the clip that 
    *                               allows for a check to happen when 
    *                               displaying the clip to make sure that the
    *                               correct clip is found
    * @param  p_clip_description    textual description of the clip created
    * @param  p_clip_title          title of the clip that will be used to
    *                               double as the title of the Portlet
    * @param  p_clip_timeout        timeout in seconds that allows the Portlet
    *                               to be timed out in the event that an HTTP
    *                               connection either could not be open or is
    *                               too slow
    * @param  p_clip_max_tries      number of times to try when timeout happens
    * @param  p_clip_expiration     the expiration time of this particular clip
    *                               in minutes. This not only stores the user
    *                               specified expiration value, but also the
    *                               kind of expiration source he desires. The
    *                               storage is agnostic to the logic of
    *                               deciding which is which.
    * @param  p_clip_rebindable     indicates whether or not this clipping
    *                               definition can be changed
    */
    procedure insert_clip(
        p_clip_id             in integer,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    );

   /**
    * This API inserts a customization for a particular clip for a particular
    * portal user id.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @param  p_url_index           index of the URL within the clipping
    *                               definition as the URLs need to be ordered
    * @param  p_param_name          name of the HTTP parameter
    * @param  p_param_value         value of the HTTP parameter
    */
    procedure insert_url_parameter(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_url_index           in integer,
        p_param_name          in varchar2,
        p_param_value         in varchar2
    );

   /**
    * This API removes all customization for a particular clip for a particular
    * portal user id.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value is the actual Portal 
    *                               user id that does the customization of
    *                               the clip created by WCR_DEFAULT_USER 
    *                               when the clip is being
    *                               created using edit defaults.
    */
    procedure remove_clip_customization(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    );
  
   /**
    * This API inserts a new browse url that is part of the original definition
    * of the Clipping. 
    *
    * This default value can be overridden by cusomized values in the
    * neighboring customized parameter table.
    *
    * @param  p_clip_id         id of the Clip that this url param is part of
    * @param  p_url_index       index of the URL within the clipping
    *                           definition as the URLs need to be ordered
    * @param  p_url             URL that was browsed to reach this clip
    * @param  p_method          method used to access the URL (GET, HEAD, ...)
    * @param  p_auth_type       HTTP Authentication type (Basic, Digest, ...)
    * @param  p_auth_realm      HTTP Authentication realm
    * @param  p_username        if needed for authentication to the URL
    * @param  p_password        if needed for authentication to the URL
    * @param  p_obfuscation_key used to encrypt the param value if the value
    *                           of p_encrypted is true
    */
    procedure insert_browse_url(
       p_clip_id           in integer,
       p_url_index         in integer,
       p_url               in varchar2,
       p_method            in varchar2,
       p_req_enc           in varchar2,
       p_auth_type         in varchar2,
       p_auth_realm        in varchar2,
       p_username          in varchar2,
       p_password          in varchar2,
       p_obfuscation_key   in varchar2
    );
  
   /**
    * This API inserts a new url param that is part of the original definition
    * of the Clipping. 
    *
    * This default value can be overridden by cusomized values in the
    * neighboring customized parameter table.
    *
    * @param  p_clip_id           Id of the Clip that this url param is part of
    * @param  p_url_index         index of the URL within the clipping
    *                             definition as the URLs need to be ordered
    * @param  p_param_name        name of the HTTP parameter
    * @param  p_param_value       value of the HTTP parameter
    * @param  param_usage         The way in which this HTTP parameter is to be
    *                             used including request parameters or headers
    * @param  p_encrypted         indicates whether this parameter is encrypted
    * @param  p_customizable      indicates whether this parameter is open for
    *                             Page Viewers to set their own values
    * @param  p_customizable_name the user-friendly input name to be used as a
    *                             label when a Page Viewer customizes this 
    *                             parameter, if the parameter is customizable
    * @param  p_obfuscation_key   used to encrypt the param value if the value
    *                             of p_encrypted is true
    */
    procedure insert_default_url_param(
       p_clip_id           in integer,
       p_url_index         in integer,
       p_param_name        in varchar2,
       p_param_value       in varchar2,
       p_param_usage       in varchar2,
       p_encrypted         in varchar2,
       p_customizable      in varchar2,
       p_customizable_name in varchar2,
       p_obfuscation_key   in varchar2
    );
  
   /** 
    * This API updates a default url parameter when the Page Designer alters 
    * the behavior of it, either to make it customizable, or to alter the
    * default value of the parameter.
    * 
    * It needs to map each parameter to a URL that is part of the clipping 
    * definition. Therefore, the URL index needs to be passed in to verify
    * the correct URL for which this is a parameter of.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @param  p_customizable        Is this parameter open for Page Viewers to
    *                               set their own values?
    * @param  p_customizable_name   If the parameter is customizable, then this
    *                               denotes the name the Page Viewer will see
    *                               when trying to set his own value
    * @param  p_url_index           index of the URL within the clipping
    *                               definition as the URLs need to be ordered
    * @param  p_param_name          name of the HTTP parameter
    * @param  p_param_value         value of the HTTP parameter to decrypt the
    *                               information that we looked up from tables
    *                               with encrypted data
    * @param  p_obfuscation_key     used to encrypt the param value if the
    *                               value of p_encrypted is true
    */
    procedure update_default_url_parameter(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_customizable        in varchar2,
        p_customizable_name   in varchar2,
        p_url_index           in integer,
        p_param_name          in varchar2,
        p_param_value         in varchar2,
        p_obfuscation_key     in varchar2
    );
    
   /**
    * This API updates the locator of the clipping for a particular user, using
    * the four keys to look up the clip id and then updating the locator table
    * using that clip id.
    *
    * This happens when fuzzy match kicks in and creates a new path
    * and signature for the current user.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @param  p_clip_path           path in HTML within the Web Page that 
    *                               allows the provider to locate the actual
    *                               clipping
    * @param  p_clip_signature      a checksum / signature of the clip that 
    *                               allows for a check to happen when 
    *                               displaying the clip to make sure that the
    *                               correct clip is found
    */
    procedure update_clip_locator(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_clip_path           in varchar2,
        p_clip_signature      in varchar2
    );
  
   /**
    * This API updates some high level attributes of the clipping definition
    * including its name, description, timeout, and rebindability.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @param  p_clip_description    Textual description of the clip created
    * @param  p_clip_title          Title of the clip that will be used to
    *                               double as the title of the Portlet
    * @param  p_clip_timeout        Timeout in seconds that allows the Portlet
    *                               to be timed out in the event that an HTTP
    *                               connection either could not be open or is
    *                               too slow
    * @param  p_clip_max_tries      number of times to try when timeout happens
    * @param  p_clip_expiration     the expiration time of this particular clip
    *                               in minutes. This not only stores the user
    *                               specified expiration value, but also the
    *                               kind of expiration source he desires. The
    *                               storage is agnostic to the logic of
    *                               deciding which is which.
    * @param  p_clip_rebindable     indicates whether or not this clipping
    *                               definition can be changed
    */
    procedure update_clip_properties(
        p_provider_id         in varchar2, 
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2, 
        p_portal_user_id      in varchar2,
        p_clip_description    in varchar2,
        p_clip_title          in varchar2,
        p_clip_timeout        in integer,
        p_clip_max_tries      in integer,
        p_clip_expiration     in integer,
        p_clip_rebindable     in varchar2
    );

   /**
    * This API removes the clip identified by a specified portlet instance
    * and the specific portal user id passed in.
    *
    * This only removes the one associated with the user passed in and
    * therefore does not remove the default (created using edit default).
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @return                       the clip id of the deleted clip
    */
    function remove_clip(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    ) return integer;
    
   /**
    * This API removes the clip identified by a specified portlet instance
    * and the specific portal user id passed in. 
    *
    * In addition, this propagates the removal down to all the customizations
    * of that clip.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    * @return                       the clip id of the deleted clip
    */
    function remove_clip_cascade(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    ) return integer;
    
   /**
    * This API removes all the clipping definitions associated with a given
    * Portlet instance in Phase I.
    *
    * This is a more complete version of the remove_clip that may remove all
    * the personalized clipping definitions for a particular Portlet instance.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    */
    procedure remove_all_clips_for_instance(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2
    );

   /**
    * In Phase II, this call is used to clean the repository when the portlet
    * definition is dropped from the provider builder.
    * 
    * @param  p_clip_id  the clip id mapping to this portlet definition
    */
    procedure remove_definition(
        p_clip_id          in integer
    );
    
   /**
    * This method is specific for phase II when a Portlet instance is removed
    * from a page.
    *
    * @param  p_provider_id         provider Id of the Web Clipping Provider 
    *                               (Usually constant) as multiple Portal 
    *                               instances can reuse the same Web Clipping
    *                               Provider
    * @param  p_portlet_id          id of the Portlet provided by the Web 
    *                               Clipping Provider, which is constant for
    *                               now (a value of 1) but for future
    *                               provisions may be changed as our next phase
    *                               entails mapping one Clipping Definition to
    *                               one Portlet Definition
    * @param  p_portlet_instance_id lowest common denominator for a clipping
    * @param  p_portal_user_id      the Portal user id that this clipping
    *                               definition is associated with. The frame-
    *                               work allows each user to have his own 
    *                               clipping definition for a given Portlet
    *                               instance. This value can either be the
    *                               actual Portal user id that is creating
    *                               the clip or it can be the user identified
    *                               by WCR_DEFAULT_USER when the clip is being
    *                               created using edit defaults.
    */
    procedure remove_instance(
        p_provider_id         in varchar2,
        p_portlet_id          in varchar2,
        p_portlet_instance_id in varchar2,
        p_portal_user_id      in varchar2
    );

   /**
    * This API encrypts p_value based on the boolean value of p_encrypted.
    *
    * @param  p_encrypted       tells if we are to be encrypted
    * @param  p_obfuscation_key the obfuscation key used to encrypt the value
    * @param  p_value           the value to encrypt
    * @return                   the encrypted form of p_value if needed
    */
    function encrypt_if_needed(
        p_encrypted       in varchar2,
        p_obfuscation_key in varchar2, 
        p_value           in varchar2
    )
    return varchar2 deterministic;

   /**
    * This API decrypts p_value based on the boolean value of p_encrypted.
    *
    * @param  p_encrypted       tells if we are to be decrypted
    * @param  p_obfuscation_key the obfuscation key used to decrypt the value
    * @param  p_value           the value to decrypt
    * @return                   the decrypted form of p_value if needed
    */
    function decrypt_if_needed(
        p_encrypted       in varchar2,
        p_obfuscation_key in varchar2, 
        p_value           in varchar2
    )
    return varchar2 deterministic;
    
   /**
    *
    */
    function get_domain(p_url in varchar2) return varchar2 deterministic;
    
   /**
    * This API inserts a new row of element name / path into the element 
    * paths table, given a specific clip id. 
    *
    * This denotes that for the given clipping there exists a scraping 
    * definition that is identified by all rows having that clip's clip id.
    *
    * @param p_clip_id      the clip id identifying the clip owning the 
    *                       scraping definition that this newly inserted 
    *                       row is part of
    * @param p_element_name the new output element name that the scraping
    *                       may output
    * @param p_element_path the path within the clipping's confines where
    *                       the output element given by p_element_name can
    *                       be found
    */
    procedure insert_element_path(
        p_clip_id              in integer,
        p_clip_element_name    in varchar2,
        p_clip_element_path    in varchar2
    );

   /**
    * This API inserts a new row of a render event given a specific clip id.
    *
    * It uses the clip id to look up information from the clip definition
    * tables to fill up this row.
    *
    * @param p_clip_id           the clip id that allows the fetching of the 
    *                            other facets of the clipping definition to 
    *                            populate the events table
    * @param p_render_status     a number that indicates either success of 
    *                            failure of the rendering call.
    * @param p_render_type       tells what type of rendering is in question,
    *                            whether it be for Portal Show Mode, for 
    *                            Wireless default SimpleResult show mode, or 
    *                            for the Wireless connector show mode
    * @param p_render_start      starting time of the rendering
    * @param p_render_end        ending time of the rendering
    * @param p_fuzzy_used        indicates whether fuzzy match was kicked in
    * @param p_db_lookup         the time it takes in milliseconds to look up
    *                            the clipping definition from the database
    * @param p_http_latency      the time it takes to reach the first byte to
    *                            read from the http remote site
    * @param p_render_user       the user (together with a possible realm) that
    *                            the rendering was done for
    * @param p_error_cause       the cause of the error if the status indicated
    *                            a failure of the rendering
    * @param p_event_description this parameter provides more information about
    *                            the logged event
    * @param p_clip_input        the input provided to render this clipping. It
    *                            is usually provided in the form of an HTTP URL
    *                            query string like abc=def
    * @param p_clip_output       the possible output (only if it's small) of 
    *                            rendering, if it gives any more hints on the
    *                            billing process.
    */
    procedure insert_render_event(
        p_clip_id           in integer,
        p_render_status     in integer,
        p_render_type       in integer,
        p_render_start      in date,
        p_render_end        in date,
        p_fuzzy_used        in integer,
        p_db_lookup         in integer,
        p_http_latency      in integer,
        p_render_user       in varchar2,
        p_error_cause       in varchar2,
        p_event_description in varchar2,
        p_clip_input        in varchar2,
        p_clip_output       in varchar2
    );

   /**
    * This API looks up rows of element name / path from the element 
    * paths table, given a specific clip id. 
    *
    * This denotes that for the given clipping there exists a scraping 
    * definition that is identified by all rows having that clip's clip id.
    *
    * @param p_clip_id          the clip id identifying the clip owning the 
    *                           scraping definition that we desire looking up
    * @param p_cv_element_paths the list of output element name / paths that
    *                           are contained within the scraping definition
    *                           of the clipping definition with the said id
    */
    procedure lookup_element_paths(
        p_clip_id              in integer,
        p_cv_element_paths     out clip_element_path_cursor
    );

   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by a specific clip id.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    *
    * @param p_clip_id          the clip id as a filtering mechanism of the
    *                           events that were logged for this particular
    *                           clip
    * @param p_cv_render_events the list of render events that are associated
    *                           with this clip
    */
    procedure lookup_render_events(
        p_clip_id          in  integer,
        p_cv_render_events out render_event_cursor
    );

   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by a specific effective url.
    *
    * This is useful for business queries that center on not the clip id but by
    * the actual location of where the clips are found. For example, if Yahoo's
    * partner, namely someone using Web Clippings, can do a search of all the
    * events that went to http://my.yahoo.com, it can find out how to charge
    * each user for what they have rendered.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    *
    * @param p_effective_url    the effective url, starting with which points
    *                           to the web page containing the clip.
    * @param p_cv_render_events the list of render events that are associated
    *                           with this clip
    */
    procedure lookup_render_events(
        p_effective_url    in  varchar2,
        p_cv_render_events out render_event_cursor
    );
    
   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by the domain.
    *
    * This is useful for business queries that center on not the clip id but by
    * the domain of where the clips are found. For example, if Yahoo can do a
    * search of all the events that went to *.yahoo.com, it can find out how to
    * charge each user for what they have rendered.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    *
    * @param p_effective_domain the domain of the effective url, starting with
    *                           which points to the web page containing the
    *                           clip. This domain is provided in the format
    *                           "oracle.com" or "yahoo.com".
    * @param p_cv_render_events the list of render events that are associated
    *                           with this clip
    */
    procedure lookup_render_events(
        p_effective_domain in  varchar2,
        p_cv_render_events out render_event_cursor
    );
    
   /**
    * This API looks up rows of render events from the wwwcp_render_log$
    * table, filtered by the user that requested the render.
    *
    * This is useful for business queries that center on not the clip id but by
    * the user to visualize how much a user has rendered, regardless of where
    * the clipping is from, so it can charge for a flat fee based on for 
    * example, the number of render events for a particular user.
    *
    * The full rendering event information structure is returned as iterable
    * by the cursor returned.
    *
    * @param p_render_user      the user on behalf of whom the render request
    *                           is made.
    * @param p_cv_render_events the list of render events that are associated
    *                           with this clip
    */
    procedure lookup_render_events(
        p_render_user      in  varchar2,
        p_cv_render_events out render_event_cursor
    );
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table.
    *
    * This is useful as a cleansing measure that an organization can do between
    * the different phases of rolling out their Web Clipping usage initiative.
    * It can be used to reset the stage before going into development testing
    * phase, or when the rollout is going live and the company wishes to begin
    * charging its subscribers for their usages, or another possible cleansing
    * may occur at for example, every month end or year end when the data
    * recorded is no longer useful.
    */
    procedure remove_all_render_events;
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table for
    * a given rendering user.
    *
    * This is useful as a cleansing measure that an organization can do for
    * example, labeling the development user to be some constant user id and
    * then removing all rendering events when it goes live.
    *
    * @param p_render_user the user on behalf of whom the render request is
    *                      made.
    */
    procedure remove_render_events(p_render_user in varchar2);
    
   /**
    * This API removes all render events from the wwwcp_render_log$ table for
    * a given starting or stopping time before or after the query time.
    *
    * This is useful as a cleansing measure that an organization can do for
    * example, all rendering events that started or stopped before a certain 
    * date are deemed development efforts and therefore should be disregarded.
    *
    * @param p_query_time   the date/time to query with
    * @param p_query_before whether the query is for before or after. A value
    *                       of 1 signifies that the query is for comparing 
    *                       times before the p_query_time while a 0 value
    *                       implies comparing times after the p_query_time.
    * @param p_query_start      whether or not the query is for start time.
    *                           A value of 1 means true while 0 means false.
    */
    procedure remove_render_events(
        p_query_time   in date,
        p_query_before in number,
        p_query_start  in number
    );
   
   /**
    * This API is a helper function that tells if a certain VARCHAR2 is
    * a number and therefore can be converted into a number. 
    * 
    * It returns a value of 1 if the given string is a number and a 
    * value of 0 if not.
    *
    * @param p_string the string in question, whether it is convertible
    *                 into a number
    * @return 1 if the p_string is a number and 0 if otherwise.
    */
    function is_number(p_string in varchar2) return integer;

end wwwcp_api_registry;
/

show errors package wwwcp_api_registry
